@@ -27,6 +27,9 @@ class Call(object):
         self.args = args
         self.kwargs = kwargs
 
+    def __str__(self):
+        return f"{self.op}({self.args}, {self.kwargs})"
+
 
 IDENT_CHAR = Regex("[@_$0-9A-Za-zÀ-ÖØ-öø-ƿ]").expr.parser_config.include
 FIRST_IDENT_CHAR = "".join(set(IDENT_CHAR) - set("0123456789"))
@@ -369,65 +372,85 @@ def to_interval_type(tokens):
             return {op: {}, **kwargs}
 
 
+def has_something(tokens, index, string):
+    if not tokens:
+        raise ParseException(
+            tokens.type, index, string, "expecting something to match"
+        ) from None
+
+
 def to_interval_call(tokens, index, string):
     # ARRANGE INTO {interval: [amount, type]} FORMAT
-    csv = list(tokens['csv'])
-    if csv:
-        seen_minute = False
-        for v in reversed(csv):
-            if seen_minute and v['type']=='minute':
-                v['type']='month'
-            if v['type'] in ['minute', 'hour', 'day', 'week']:
-                seen_minute = True
+    expr = tokens["expr"]
+    type = tokens["type"]
 
-        result = Call("add", [
-            Call("interval", [v['expr'], v['type']], {})
-            for v in csv
-        ], {})
-    else:
-        type = tokens['type']
-        formatted = dict(tokens['formatted'])
-        expr = tokens["expr"]
+    if expr and type:
+        return Call("interval", [expr, type], {})
+    if expr:
+        return Call("interval", [expr], {})
+    ago = -1 if tokens["ago"] else 1
+    durations = {
+        k: v for k, v in dict(tokens).items() if k not in ("type", "ago", "day-ago")
+    }
+    if tokens["day"] and ago == -1 and tokens["day-ago"] == "+":
+        durations["day"] *= -1
+    result = Call(
+        "add", [Call("interval", [ago * v, k], {}) for k, v in durations.items()], {},
+    )
 
-        if formatted:
-            if 'year' in formatted:
-                if len(formatted) == 1:
-                    return Call("interval", [formatted['year'], type or 'second'], {})
-                elif 'year' not in type and 'month' in type:
-                    formatted['day'] = formatted['month']
-                    formatted['month'] = formatted['year']
-                    formatted['year'] = None
-            if 'hour' in formatted:
-                if 'hour' not in type and "minute" in type:
-                    formatted['second'] = formatted['minute']
-                    formatted['minute'] = formatted['hour']
-                    formatted['hour'] = None
-            if 'second' in formatted and 'fraction' in formatted and len(formatted) == 2:
-                if not type:
-                    raise ParseException(
-                        tokens['formatted'].type,
-                        tokens['formatted'].start,
-                        string,
-                        """Ambiguious value for given interval""",
-                    )
-                return Call("interval", [float(str(formatted['second'])+"."+str(formatted['fraction'])), type], {})
+    if len(result.args) == 1:
+        result = result.args[0]
 
-            result = Call("add", [
-                Call("interval", [v, k], {})
-                for k, v in formatted.items()
-                if v is not None
-            ], {})
-        elif expr:
+    if type:
+        return Call("cast", [result, type], {})
+
+    return result
+
+
+def cast_interval_call(tokens, index, string):
+    # ARRANGE INTO {interval: [amount, type]} FORMAT
+    expr = tokens["expr"]
+    type = tokens["type"]
+
+    while isinstance(expr, ParseResults) and expr.length() == 1:
+        expr = expr[0]
+
+    if isinstance(expr, (int, float, str, dict)):
+        return Call("interval", [expr, type or "second"], {})
+
+    if isinstance(expr, Call):
+        if (
+            expr.op == "interval"
+            or expr.op == "add"
+            and all(isinstance(e, Call) and e.op == "interval" for e in expr.args)
+        ):
+            if type:
+                return Call("cast", [expr, type], {})
+            return expr
+
+        type = tokens["type"]
+        if type:
             return Call("interval", [expr, type], {})
-        else:
-            # SIMPLE TYPE, NO VALUE
-            return type
+        return Call("interval", [expr], {})
 
-    # SIMPLIFY
-    if len(result.args) > 1:
-        return result
-    return result.args[0]
+    acc = []
+    for e in list(expr):
+        if isinstance(e, Call):
+            if e.op == "add":
+                acc.extend(e.args)
+            else:
+                acc.append(e)
+    if len(acc) == 1:
+        expr = acc[0]
+    else:
+        expr = Call("add", args=acc, kwargs={})
 
+    if expr.op == "interval" and len(expr.args) == 1:
+        expr.args.append(type or "second")
+        return expr
+    if type:
+        return Call("cast", [expr, type], {})
+    return expr
 
 
 def to_case_call(tokens):
@@ -752,6 +775,10 @@ real_num = (
     Regex(r"[+-]?(\d+\.\d*|\.\d+)([eE][+-]?\d+)?").set_parser_name("float")
     / (lambda t: float(t[0]))
 )
+real_pos = (
+    Regex(r"(\d+\.\d*|\.\d+)([eE][+-]?\d+)?").set_parser_name("float")
+    / (lambda t: float(t[0]))
+)
 
 
 def parse_int(tokens):
@@ -762,6 +789,7 @@ def parse_int(tokens):
 
 
 int_num = Regex(r"[+-]?\d+([eE]\+?\d+)?").set_parser_name("int") / parse_int
+int_pos = Regex(r"\d+([eE]\+?\d+)?").set_parser_name("int") / parse_int
 hex_num = (
     Regex(r"0x[0-9a-fA-F]+").set_parser_name("hex") / (lambda t: {"hex": t[0][2:]})
 )
